# (C) Copyright 2004-2021 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

""" The organization in this package should more or less mirror that of
the ``traitsui.wx`` package and contains the corresponding logic for testing.

Note that once new implementations are added, they may need to be exposed
via the ``default_registry`` module (one level up from this package).
"""
