# (C) Copyright 2004-2021 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

""" Defines the concrete implementations of the traits Toolkit interface for
    the wxPython user interface toolkit.
"""

# -------------------------------------------------------------------------
#  Define the reference to the exported GUIToolkit object:
# -------------------------------------------------------------------------

from . import toolkit

# Reference to the GUIToolkit object for wxPython
toolkit = toolkit.GUIToolkit("traitsui", "wx", "traitsui.wx")
